/*
 * Copyright 2001-2005, Broadcom Corporation
 * All Rights Reserved.
 * 
 * THIS SOFTWARE IS OFFERED "AS IS", AND BROADCOM GRANTS NO WARRANTIES OF ANY
 * KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE. BROADCOM
 * SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE.
 *
 * $Id$
 *
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <net/route.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <time.h>

#include <bcmnvram.h>
#include <netconf.h>
#include <shutils.h>
#include <rc.h>
#include "./gemtek_added/Peter_Added_002.h"
#include "./gemtek_added/Peter_Added_For_Restart_All_Apps_DHCP_Mode.h"

#define IFUP (IFF_UP | IFF_RUNNING | IFF_BROADCAST | IFF_MULTICAST)

static int
expires(unsigned int in)
{
	time_t now;
	FILE *fp;

	time(&now);
	if (!(fp = fopen("/tmp/udhcpc.expires", "w"))) {
		perror("/tmp/udhcpd.expires");
		return errno;
	}
	fprintf(fp, "%d", (unsigned int) now + in);
	fclose(fp);
	return 0;
}	

/* 
 * deconfig: This argument is used when udhcpc starts, and when a
 * leases is lost. The script should put the interface in an up, but
 * deconfigured state.
*/
static int
deconfig(void)
{
	char *wan_ifname = safe_getenv("interface");

	ifconfig(wan_ifname, IFUP,
		 "0.0.0.0", NULL);
	expires(0);
	Peter_Added_002_Extra_SRCs_001

	dprintf("done\n");
	return 0;
}

/*
 * bound: This argument is used when udhcpc moves from an unbound, to
 * a bound state. All of the paramaters are set in enviromental
 * variables, The script should configure the interface, and set any
 * other relavent parameters (default gateway, dns server, etc).
*/
static int		bound ( void )
{
	char		*wan_ifname = safe_getenv ( "interface" );
	char		*value = NULL;
	char		*wandhcp_dns = NULL;
	int		StringLen = 0;

	Peter_Added_002_Extra_SRCs_002            /*Walter: control Wan LED */
	Restart_Apps_DHCP_Mode_Extra_SRCs_001     /*Walter: declare tmp variable */
	Restart_Apps_DHCP_Mode_Extra_SRCs_002
	if ( ( value = getenv ( "ip" ) ) ) {
		nvram_set("wan_ipaddr", value);
	}

	if ((value = getenv("subnet")))
		nvram_set("wan_netmask", value);
        if ((value = getenv("router")))
		nvram_set("wan_gateway", value);

	wandhcp_dns = nvram_get ( "wandhcp_dns" );
	if ( wandhcp_dns == NULL ) {
AAA:		if ( ( value = getenv ( "dns" ) ) ) {
			nvram_set ( "wan_dns" , value );
		}
	} else {
		StringLen = strlen ( wandhcp_dns );
		if ( ( StringLen == 0 ) || ( StringLen < 0 ) ) {
			goto	AAA;
		} else {
			if ( ( *wandhcp_dns >= '1' ) && ( *wandhcp_dns <= '9' ) ) {
				nvram_set ( "wan_dns" , wandhcp_dns );
			} else {
				goto	AAA;
			}
		}
	}

	if ((value = getenv("wins")))
		nvram_set("wan_wins", value);
	if ((value = getenv("hostname")))
	{
/*		char *hostname = nvram_get("wan_hostname");
		if ( hostname == NULL || !strcmp(hostname,""))
*/			nvram_set("wan_hostname", value);
	}
	if ((value = getenv("domain")))
	{
/*		char *domain = nvram_get("wan_domain");
		if ( domain == NULL || !strcmp(domain,""))
*/			nvram_set("wan_domain", value);
	}
	if ((value = getenv("lease"))) {
		nvram_set("wan_lease", value);
		expires(atoi(value));
	}

	ifconfig ( wan_ifname , IFUP , nvram_safe_get ( "wan_ipaddr" ) , nvram_safe_get ( "wan_netmask" ) );
	start_wan_done ();

	Restart_Apps_DHCP_Mode_Extra_SRCs_003

	Peter_Added_002_Extra_SRCs_002
	dprintf ( "done\n" );
	return	0;
}

/*
 * renew: This argument is used when a DHCP lease is renewed. All of
 * the paramaters are set in enviromental variables. This argument is
 * used when the interface is already configured, so the IP address,
 * will not change, however, the other DHCP paramaters, such as the
 * default gateway, subnet mask, and dns server may change.
 */
static int
renew(void)
{
	stop_firewall();
	bound();

	dprintf("done\n");
	return 0;
}

int
udhcpc_main(int argc, char **argv)
{
	if (!argv[1])
		return EINVAL;
	else if (strstr(argv[1], "deconfig"))
		return deconfig();
	else if (strstr(argv[1], "bound"))
		return bound();
	else if (strstr(argv[1], "renew"))
		return renew();
	else
		return EINVAL;
}
